@@ -8,11 +8,16 @@ import base_io
 from . import _set_vs, get_version, ForbiddenError, UnauthorizedError, \
         NotFoundError, Redirect
 
-# TODO: an easy method for building a design doc from a module
-# TODO: docstrings
-
 class BasePythonViewServer(base_io.BaseViewServer):
+    """Python view server logic, with an overridable compile() method"""
+
     def __init__(self, stdin, stdout):
+        """
+        stdin, stdout: where to read and write data
+
+        warning: they should be opened in 'line buffered' or 'unbuffered' mode
+        """
+
         super(BasePythonViewServer, self).__init__(stdin, stdout)
         self.ddocs = {}
         self.reset(silent=True)
@@ -24,6 +29,7 @@ class BasePythonViewServer(base_io.BaseViewServer):
 
     def use_ddoc(self, doc_id, func_path, func_args):
         """Call a function of a previously added ddoc"""
+
         assert doc_id in self.ddocs
         (doc, cache) = self.ddocs[doc_id]
 
@@ -46,11 +52,12 @@ class BasePythonViewServer(base_io.BaseViewServer):
         getattr(self, "ddoc_" + func_type)(func, func_args)
 
     def ddoc_shows(self, func, args):
+        """execute a show function"""
+
         (doc, req) = args
 
         _set_vs(self, ["start", "send", "log"])
-        self.response_start = None
-        self.chunks = []
+        self._clear_state()
 
         try:
             value = func(doc, req)
@@ -88,24 +95,80 @@ class BasePythonViewServer(base_io.BaseViewServer):
 
             self.output("resp", value)
 
-        self.response_start = None
-        self.chunks = []
-
         _set_vs(None)
+        self._clear_state()
 
     def ddoc_lists(self, func, args):
-        pass # TODO ddoc_lists
+        """execute a list function"""
+        (head, req) = args
+
+        _set_vs(self, ["start", "send", "get_row", "log"])
+        self._clear_state()
+
+        tail = None
+
+        try:
+            if inspect.isgeneratorfunction(func):
+                g = func(head, req, self._get_row_generator())
+                for y in g:
+                    if isinstance(y, dict):
+                        self.start(y)
+                    else:
+                        assert isinstance(y, basestring)
+                        self.send(y)
+            else:
+                tail = func(head, req)
+
+        except NotFoundError as e:
+            msg = str(e)
+            if not msg:
+                msg = "document not found"
+            self.output("error", "not_found", msg)
+            return
+
+        except Redirect as e:
+            if e.permanent:
+                c = 301
+            else:
+                c = 302
+            # self.start will assert that start hasn't already been sent
+            self.start({"code": c, "headers": {"Location": e.url}})
+
+        if tail != None:
+            self.send(tail)
+
+        if not self.have_sent_start:
+            self.get_row() # And discard
+
+        self._send_list_chunks("end")
+
+        self._clear_state()
+        _set_vs(None)
 
     def ddoc_filters(self, func, args):
+        """execute a filter function"""
+
         (docs, req) = args
         _set_vs(self, ["log"])
         self.output(True, [bool(func(doc, req)) for doc in docs])
         _set_vs(None)
 
     def ddoc_updates(self, func, args):
-        pass # TODO ddoc_updates
+        """execute an update function"""
+
+        (doc, req) = args
+        _set_vs(self, ["log"])
+        (doc, response) = func(doc, req)
+        _set_vs(None)
+
+        if isinstance(response, basestring):
+            response = {"body": response}
+
+        self.output("up", doc, response);
 
     def ddoc_validate_doc_update(self, func, args):
+        """execute a validate_doc_update function"""
+
         assert len(args) == 4 # newdoc, olddoc, userctx, secobj
 
         _set_vs(self, ["log"])
@@ -122,21 +185,66 @@ class BasePythonViewServer(base_io.BaseViewServer):
         _set_vs(None)
 
     def start(self, response_start):
+        """the start() callback from show functions"""
         assert self.response_start == None
+        assert not self.have_sent_start
         self.response_start = response_start
 
     def send(self, chunk):
+        """the send() callback from show functions"""
         self.chunks.append(chunk)
 
     def get_row(self):
-        pass # TODO get_row
+        """the get_row() callback from list functions"""
+        if self.list_ended:
+            return None
+
+        if not self.have_sent_start:
+            self._send_list_start()
+        else:
+            self._send_list_chunks()
+
+        obj = self.read_line()
+        assert obj and obj[0] in ["list_row", "list_end"]
+
+        if obj[0] == "list_end":
+            self.list_ended = True
+            return None
+        else:
+            return obj[1]
+
+    def _get_row_generator(self):
+        while True:
+            row = self.get_row()
+            if row == None:
+                break
+            yield row
+
+    def _send_list_start(self):
+        if self.response_start == None:
+            self.response_start = {}
+        self.output("start", self.chunks, self.response_start)
+        self.chunks = []
+        self.have_sent_start = True
+        self.response_start = None
+
+    def _send_list_chunks(self, label="chunks"):
+        """empty self.chunks by sending them to couch"""
+        self.output(label, self.chunks)
+        self.chunks = []
+
+    def _clear_state(self):
+        """clear request specific state (emit, send, etc)"""
+        self.emissions = []
+        self.response_start = None
+        self.chunks = []
+        self.have_sent_start = False
+        self.list_ended = False
 
     def reset(self, config=None, silent=False):
         """Reset state and garbage collect. Apply config, if present"""
+
         self.map_funcs = []
-        self.emissions = None
-        self.response_start = None
-        self.chunks = None
         self.view_ddoc = {}
         if config:
             self.query_config = config
@@ -157,13 +265,14 @@ class BasePythonViewServer(base_io.BaseViewServer):
 
     def map_doc(self, doc):
         """run all map functions on a document"""
+
         assert self.map_funcs
 
         _set_vs(self, ["emit", "log"])
         results = []
 
         for func in self.map_funcs:
-            self.emissions = []
+            self._clear_state()
 
             try:
                 if inspect.isgeneratorfunction(func):
@@ -179,16 +288,27 @@ class BasePythonViewServer(base_io.BaseViewServer):
                 results.append(self.emissions)
 
         _set_vs(None)
-        self.emissions = None
+        self._clear_state()
 
         self.output(*results)
 
     def emit(self, key, value):
-        """callback from map functions"""
+        """the emit() callback from map functions"""
         self.emissions.append([key, value])
 
+    def _reduce_limit(self):
+        """calculate the reduce limit size"""
+        if "reduce_limit" not in self.query_config:
+            return None
+        if not self.query_config["reduce_limit"]:
+            return None
+
+        i = self._input_line_length
+        return max(200, i / 2)
+
     def reduce(self, funcs, data):
         """run reduce functions on some data"""
+
         # data is [[key, id], value].
         keys = [i[0] for i in data]
         values = [i[1] for i in data]
@@ -207,11 +327,11 @@ class BasePythonViewServer(base_io.BaseViewServer):
 
         _set_vs(None)
 
-        # TODO: self.query_config["reduce_limit"]
-        self.output(True, results)
+        self.output(True, results, limit=self._reduce_limit())
 
     def rereduce(self, funcs, values):
         """run reduce functions on some reduce function outputs"""
+
         results = []
 
         _set_vs(self, ["log"])
@@ -228,14 +348,15 @@ class BasePythonViewServer(base_io.BaseViewServer):
 
         _set_vs(None)
 
-        # TODO: self.query_config["reduce_limit"]
-        self.output(True, results)
+        self.output(True, results, limit=self._reduce_limit())
 
     def compile(self, function):
         """produce something that can be executed, from a string"""
         raise NotImplementedError
 
 class NamedPythonViewServer(BasePythonViewServer):
+    """python server that 'compiles' functions by importing the given path"""
+
     def compile(self, function):
         """import a function by name"""
         try:
@@ -265,6 +386,7 @@ class NamedPythonViewServer(BasePythonViewServer):
         return f
 
 def main():
+    """main function for couch-named-python"""
     linebuf_in = os.fdopen(sys.stdin.fileno(), 'r', 1)
     linebuf_out = os.fdopen(sys.stdout.fileno(), 'w', 1)
 
